@@ -156,11 +156,7 @@ public:
     };
 #endif
 
-#ifdef UNIT_TEST
 public:
-#else
-private:
-#endif
     typedef ::std::vector<data_type*> data_chain_type;
     typedef _mdds_unordered_map_type<data_type*, ::std::pair<key_type, key_type> > segment_map_type;
     typedef ::std::map<data_type*, ::std::pair<key_type, key_type> >               sorted_segment_map_type;
@@ -234,6 +230,7 @@ private:
     {
         ::std::string operator() (const node& _self) const
         {
+#ifdef UNIT_TEST
             ::std::ostringstream os;
             if (_self.is_leaf)
             {
@@ -260,6 +257,9 @@ private:
             }
             os << " ";
             return os.str();
+#else
+            return ::std::string();
+#endif
         }
     };
 
@@ -436,9 +436,18 @@ private:
 
         bool operator== (const iterator_base& r) const
         {
-            return mp_res_chains.get() == r.mp_res_chains.get() && 
-                m_cur_chain == r.m_cur_chain && m_cur_pos_in_chain == r.m_cur_pos_in_chain &&
-                m_end_pos == r.m_end_pos;
+            if (mp_res_chains.get())
+            {
+                // non-empty result set.
+                return mp_res_chains.get() == r.mp_res_chains.get() &&
+                    m_cur_chain == r.m_cur_chain && m_cur_pos_in_chain == r.m_cur_pos_in_chain &&
+                    m_end_pos == r.m_end_pos;
+            }
+
+            // empty result set.
+            if (r.mp_res_chains.get())
+                return false;
+            return m_end_pos == r.m_end_pos;
         }
 
         bool operator!= (const iterator_base& r) const { return !operator==(r); }
