@@ -25,10 +25,8 @@
  *
  ************************************************************************/
 
-#if 0 // Disabled until I re-write priority search tree in a template.
-#include "rangetree.hpp"
-#endif
 #include "mdds/flat_segment_tree.hpp"
+#include "test_global.hpp"
 
 #include <list>
 #include <iostream>
@@ -42,49 +40,6 @@
 using namespace std;
 using namespace mdds;
 
-
-#include <stdio.h>
-#include <string>
-#include <sys/time.h>
-
-namespace {
-
-class StackPrinter
-{
-public:
-    explicit StackPrinter(const char* msg) :
-        msMsg(msg)
-    {
-        fprintf(stdout, "%s: --begin\n", msMsg.c_str());
-        mfStartTime = getTime();
-    }
-
-    ~StackPrinter()
-    {
-        double fEndTime = getTime();
-        fprintf(stdout, "%s: --end (duration: %g sec)\n", msMsg.c_str(), (fEndTime-mfStartTime));
-    }
-
-    void printTime(int line) const
-    {
-        double fEndTime = getTime();
-        fprintf(stdout, "%s: --(%d) (duration: %g sec)\n", msMsg.c_str(), line, (fEndTime-mfStartTime));
-    }
-
-private:
-    double getTime() const
-    {
-        timeval tv;
-        gettimeofday(&tv, NULL);
-        return tv.tv_sec + tv.tv_usec / 1000000.0;
-    }
-
-    ::std::string msMsg;
-    double mfStartTime;
-};
-
-}
-
 void printTitle(const char* msg)
 {
     cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
@@ -92,157 +47,6 @@ void printTitle(const char* msg)
     cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
 }
 
-#if 0 // Disabled until I re-write priority search tree in a template.
-void testPrioSearchTree()
-{
-    RangeTree emptyDB;
-    emptyDB.insertPoint("Raleigh", 90, 34);
-    emptyDB.build();
-//  emptyDB.dump();
-    emptyDB.search(0, 100);
-    emptyDB.search(10, 20);
-
-    RangeTree rangeDB;
-    rangeDB.insertPoint("North Pole", -35, -42);
-    rangeDB.insertPoint("Chicago", 35, 42);
-    rangeDB.insertPoint("Mobile", 52, 10);
-    rangeDB.insertPoint("Toronto", 62, 77);
-    rangeDB.insertPoint("Buffalo", 82, 65);
-    rangeDB.insertPoint("Denver", 5, 45);
-    rangeDB.insertPoint("Provo", 5, 46);
-    rangeDB.insertPoint("Provo", 5, 46);
-    rangeDB.insertPoint("Provo", 5, 46);
-    rangeDB.insertPoint("Provo", 5, 46);
-    rangeDB.insertPoint("Omaha", 27, 35);
-    rangeDB.insertPoint("Atlanta", 85, 15);
-    rangeDB.insertPoint("Miami", 90, 5);
-    rangeDB.insertPoint("Raleigh", 90, 34);
-    rangeDB.insertPoint("New York", 102, 44);
-    rangeDB.insertPoint("Seattle", 2, 1029);
-    rangeDB.insertPoint("Bemidji", 23, 113);
-    rangeDB.build();
-//  rangeDB.dump();
-
-    rangeDB.search(30, 200);
-    rangeDB.printSearchResult();
-    rangeDB.search(0, 52);
-    rangeDB.printSearchResult();
-    rangeDB.search(102, 150);
-    rangeDB.printSearchResult();
-
-    rangeDB.search(0, 100, 30);
-    rangeDB.printSearchResult();
-    rangeDB.search(0, 50, 100);
-    rangeDB.printSearchResult();
-    rangeDB.search(40, 100, 10);
-    rangeDB.printSearchResult();
-    rangeDB.search(-100, 10000, -100);
-    rangeDB.printSearchResult();
-    rangeDB.search(-100, -99, 1000);
-    rangeDB.printSearchResult();
-}
-
-void testPrioSearchTree2()
-{
-    RangeTree db;
-    for (int x = 0; x < 10; ++x)
-        for (int y = 0; y < 10; ++y)
-            db.insertPoint("point", x, y);
-    db.build();
-
-    for (int x = 0; x < 4; ++x)
-    {
-        db.search(x, x+1, 0);
-        db.printSearchResult();
-    }
-
-    for (int x = 0; x < 4; ++x)
-        for (int y = 0; y < 5; ++y)
-        {
-            db.search(x, x+1, y);
-            db.printSearchResult();
-        }
-}
-
-namespace {
-
-struct SegmentName : public segment_tree::segment_data_type
-{
-    string name;
-    SegmentName(const char* _name) :
-        name(_name) {}
-
-    virtual ~SegmentName() {}
-
-    virtual size_t hash() const
-    {
-        size_t n = name.size();
-        if (n > 10)
-            // Put a cap at 10 characters.
-            n = 10;
-
-        size_t hash = 0;
-        for (size_t i = 0; i < n; ++i)
-        {
-            char c = name[i];
-            hash += static_cast<size_t>(c);
-        }
-        return hash;
-    }
-
-    virtual const char* what() const
-    {
-        return name.c_str();
-    }
-};
-
-}
-
-void testSegmentTree1()
-{
-    struct {
-        segment_tree::value_type  low;
-        segment_tree::value_type  high;
-        SegmentName*            name;
-    } points[] = {
-        { 6, 36, new SegmentName("A")},
-        {34, 38, new SegmentName("B")},
-        {21, 36, new SegmentName("C")},
-        {23, 27, new SegmentName("D")},
-        { 3,  8, new SegmentName("E")},
-        {15, 19, new SegmentName("F")},
-        {11, 14, new SegmentName("G")}
-    };
-
-    segment_tree db;
-    size_t n = sizeof(points)/sizeof(points[0]);
-    for (size_t i = 0; i < n; ++i)
-    {
-        cout << "inserting segment into tree: " << points[i].low << " - " << points[i].high << " ("
-             << points[i].name->what() << ")" << endl;
-        db.insert_front(points[i].low, points[i].high, points[i].name);
-    }
-
-    db.build();
-    for (size_t i = 0; i < 40; ++i)
-    {
-        list<const segment_tree::segment_data_type*> hits;
-        db.search(i, hits);
-        cout << "segments that contains point " << i << ":";
-        for (list<const segment_tree::segment_data_type*>::const_iterator itr = hits.begin(), itrEnd = hits.end();
-              itr != itrEnd; ++itr)
-        {
-            const segment_tree::segment_data_type* p = *itr;
-            cout << " " << static_cast<const SegmentName*>(p)->name;
-        }
-        cout << endl;
-    }
-
-    for (size_t i = 0; i < n; ++i)
-        delete points[i].name;
-}
-#endif
-
 void fst_test_leaf_search()
 {
     {
@@ -312,7 +116,7 @@ void fst_test_leaf_search()
         for (int i = 0; i <= 100; ++i)
         {
             int val = 0;
-            if (db.search(i, val))
+            if (db.search(i, val).second)
                 cout << "key = " << i << "; value = " << val << endl;
             else
                 cout << "key = " << i << "; (value not found)" << endl;
@@ -320,7 +124,7 @@ void fst_test_leaf_search()
         for (int i = 0; i <= 100; ++i)
         {
             int val = 0, start, end;
-            if (db.search(i, val, &start, &end))
+            if (db.search(i, val, &start, &end).second)
                 cout << "key = " << i << "; value = " << val << "(span: " << start << " - " << end << ")" << endl;
             else
                 cout << "key = " << i << "; (value not found)" << endl;
@@ -396,7 +200,7 @@ void fst_perf_test_search(bool tree_search)
             }
             else
             {
-                if (db.search(i, val))
+                if (db.search(i, val).second)
                     ++success;
                 else
                     ++failure;
@@ -657,7 +461,7 @@ void fst_test_insert_search_mix()
     test_single_tree_search(db, 30, 999, 0, 100);
 }
 
-void fst_test_shift_segment_left()
+void fst_test_shift_left()
 {
     StackPrinter __stack_printer__("fst_test_shift_segment_left");
     typedef flat_segment_tree<int, int> db_type;
@@ -806,7 +610,7 @@ void fst_test_shift_segment_left()
     build_and_dump(db);
 }
 
-void fst_test_shift_segment_left_right_edge()
+void fst_test_shift_left_right_edge()
 {
     StackPrinter __stack_printer__("fst_test_shift_segment_left_right_edge");
     flat_segment_tree<int, bool> db(0, 100, false);
@@ -870,7 +674,7 @@ void fst_test_shift_segment_left_right_edge()
     }
 }
 
-void fst_test_shift_segment_left_append_new_segment()
+void fst_test_shift_left_append_new_segment()
 {
     StackPrinter __stack_printer__("fst_test_shift_segment_left_append_new_segment");
     flat_segment_tree<int, bool> db(0, 100, false);
@@ -915,7 +719,7 @@ void fst_test_shift_segment_left_append_new_segment()
     }
 }
 
-void fst_test_shift_segment_right_init0()
+void fst_test_shift_right_init0()
 {
     StackPrinter __stack_printer__("fst_test_shift_segment_right_init0");
 
@@ -1040,7 +844,7 @@ void fst_test_shift_segment_right_init0()
     }
 }
 
-void fst_test_shift_segment_right_init999()
+void fst_test_shift_right_init999()
 {
     StackPrinter __stack_printer__("fst_test_shift_segment_right_init999");
 
@@ -1087,7 +891,7 @@ void fst_test_shift_segment_right_init999()
     assert(db.is_tree_valid());
 }
 
-void fst_test_shift_segment_right_bool()
+void fst_test_shift_right_bool()
 {
     flat_segment_tree<long, bool> db(0, 1048576, false);
     db.insert_front(3, 7, true);
@@ -1109,7 +913,7 @@ void fst_test_shift_segment_right_bool()
     }
 }
 
-void fst_test_shift_segment_right_skip_start_node()
+void fst_test_shift_right_skip_start_node()
 {
     StackPrinter __stack_printer__("fst_test_shift_segment_right_skip_start_node");
 
@@ -1131,6 +935,58 @@ void fst_test_shift_segment_right_skip_start_node()
         short v[] = {0, 5, 0};
         assert(check_leaf_nodes(db, k, v, ARRAY_SIZE(k)));
     }
+
+    // shift_right from the leftmost node should not change its value
+    db.insert_front(0, 4, 2);
+    assert(!db.is_tree_valid());
+    build_and_dump(db);
+    {
+        long  k[] = {0, 4, 9, 1048576};
+        short v[] = {2, 5, 0};
+        assert(check_leaf_nodes(db, k, v, ARRAY_SIZE(k)));
+    }
+
+    db.shift_right(0, 2, true);
+    build_and_dump(db);
+    {
+        long  k[] = {0, 6, 11, 1048576};
+        short v[] = {2, 5, 0};
+        assert(check_leaf_nodes(db, k, v, ARRAY_SIZE(k)));
+    }
+}
+
+/**
+ * Right all nodes right so that all existing nodes get pushed out of the
+ * range.
+ */
+void fst_test_shift_right_all_nodes()
+{
+    typedef flat_segment_tree<unsigned, unsigned> fst_type;
+
+    fst_type db(0, 10, 0);
+    {
+        unsigned k[] = {0, 10};
+        unsigned v[] = {0};
+        assert(check_leaf_nodes(db, k, v, ARRAY_SIZE(k)));
+    }
+
+    db.insert_back(0, 8, 2);
+    db.dump_leaf_nodes();
+    {
+        unsigned k[] = {0, 8, 10};
+        unsigned v[] = {2, 0};
+        assert(check_leaf_nodes(db, k, v, ARRAY_SIZE(k)));
+    }
+
+    // Shift all nodes out of range.  After this, there should be only the
+    // left and right most nodes left.
+    db.shift_right(0, 15, false);
+    db.dump_leaf_nodes();
+    {
+        unsigned k[] = {0, 10};
+        unsigned v[] = {0};
+        assert(check_leaf_nodes(db, k, v, ARRAY_SIZE(k)));
+    }
 }
 
 template<typename key_type, typename value_type>
@@ -1186,6 +1042,7 @@ template<typename key_type, typename value_type>
 void fst_test_insert_front_back(key_type start_key, key_type end_key, value_type default_value)
 {
     typedef flat_segment_tree<key_type, value_type> container_type;
+    typedef typename container_type::const_iterator itr_type;
 
     value_type val = 0;
 
@@ -1193,7 +1050,9 @@ void fst_test_insert_front_back(key_type start_key, key_type end_key, value_type
     container_type db_front(start_key, end_key, default_value);
     for (key_type i = start_key; i < end_key - 10; ++i)
     {
-        db_front.insert_front(i, i+1, val);
+        itr_type itr = db_front.insert_front(i, i+1, val).first;
+        assert(itr->first == i);
+        assert(itr->second == val);
         if (++val > 10)
             val = 0;
     }
@@ -1203,7 +1062,9 @@ void fst_test_insert_front_back(key_type start_key, key_type end_key, value_type
     val = 0;
     for (key_type i = start_key; i < end_key - 10; ++i)
     {
-        db_back.insert_back(i, i+1, val);
+        itr_type itr = db_back.insert_back(i, i+1, val).first;
+        assert(itr->first == i);
+        assert(itr->second == val);
         if (++val > 10)
             val = 0;
     }
@@ -1219,7 +1080,7 @@ void fst_test_insert_front_back(key_type start_key, key_type end_key, value_type
     }
 }
 
-void fst_perf_test_insert()
+void fst_perf_test_insert_front_back()
 {
     typedef unsigned long key_type;
     typedef int           value_type;
@@ -1361,6 +1222,14 @@ void fst_test_equality()
     db2.insert_front(0, 1, 1);
     assert(db1 == db2);
 
+    // Same node count, but with different value.
+    db2.insert_front(0, 1, 2);
+    assert(db1 != db2);
+
+    // Set the value back.
+    db2.insert_front(0, 1, 1);
+    assert(db1 == db2);
+
     db1.insert_front(4, 10, 10);
     db1.insert_front(4, 10, 0);
     assert(db1 == db2);
@@ -1441,56 +1310,597 @@ void fst_test_back_insert()
     db.dump_leaf_nodes();
 }
 
-int main (int argc, char *argv[])
+template<typename A, typename B>
+void print_iterator(typename flat_segment_tree<A,B>::const_iterator& itr)
 {
-#if 0 // Disabled until I re-write priority search tree in a template.
-    testPrioSearchTree();
-    testPrioSearchTree2();
-    testSegmentTree1();
-#endif
+    cout << "iterator: (key=" << itr->first << ",value=" << itr->second << ")" << endl;
+}
 
-    // ------------------------------------------------------------------------
-    // flat_segment_tree test
+void fst_test_insert_iterator()
+{
+    StackPrinter __stack_printer__("::fst_test_insert_iterator");
+    typedef long key_type;
+    typedef short value_type;
+    typedef flat_segment_tree<key_type, value_type> db_type;
 
-//  fst_perf_test_insert();
-    fst_test_equality();
-    fst_test_copy_ctor();
-    fst_test_back_insert();
+    db_type db(0, 100, 0);
+    db_type::const_iterator itr;
+
+    itr = db.insert_front(0, 5, 4).first;
+    assert(itr->first == 0);
+    assert(itr->second == 4);
+    print_iterator<key_type, value_type>(itr);
+
+    itr = db.insert_front(3, 10, 100).first;
+    assert(itr->first == 3);
+    assert(itr->second == 100);
+    print_iterator<key_type, value_type>(itr);
+
+    itr = db.insert_front(5, 8, 100).first;
+    assert(itr->first == 3);
+    assert(itr->second == 100);
+    print_iterator<key_type, value_type>(itr);
+
+    itr = db.insert_front(5, 8, 50).first;
+    assert(itr->first == 5);
+    assert(itr->second == 50);
+    print_iterator<key_type, value_type>(itr);
+
+    itr = db.insert_front(6, 9, 50).first;
+    assert(itr->first == 5);
+    assert(itr->second == 50);
+    print_iterator<key_type, value_type>(itr);
+
+    itr = db.insert_front(9, 20, 24).first;
+    assert(itr->first == 9);
+    assert(itr->second == 24);
+    print_iterator<key_type, value_type>(itr);
+
+    itr = db.insert_front(19, 24, 34).first;
+    assert(itr->first == 19);
+    assert(itr->second == 34);
+    print_iterator<key_type, value_type>(itr);
+
+    itr = db.insert_front(24, 26, 0).first;
+    assert(itr->first == 24);
+    assert(itr->second == 0);
+    print_iterator<key_type, value_type>(itr);
+
+    itr = db.insert_front(30, 50, 2).first;
+    assert(itr->first == 30);
+    assert(itr->second == 2);
+    print_iterator<key_type, value_type>(itr);
+
+    itr = db.insert_front(120, 140, 34).first;
+    assert(itr == db.end());
+
+    itr = db.insert_front(-20, -10, 20).first;
+    assert(itr == db.end());
+}
+
+void fst_test_insert_state_changed()
+{
+    StackPrinter __stack_printer__("::fst_test_insert_state_changed");
+    typedef long key_type;
+    typedef short value_type;
+    typedef flat_segment_tree<key_type, value_type> db_type; 
+    typedef pair<db_type::const_iterator, bool> ret_type;
+
+    db_type db(0, 1000, 0);
+
+    // Inserting a segment with the default value.  This should not change the
+    // state.
+    ret_type r = db.insert_front(10, 15, 0);
+    assert(!r.second);
+
+    // New segment with a different value.
+    r = db.insert_front(0, 10, 1);
+    assert(r.second);
+
+    // Inserting the same segment should not change the state.
+    r = db.insert_front(0, 10, 1);
+    assert(!r.second);
+
+    r = db.insert_front(0, 1, 1);
+    assert(!r.second);
+
+    r = db.insert_front(8, 10, 1);
+    assert(!r.second);
+
+    // This extends the segment, therefore the state should change.
+    r = db.insert_front(8, 11, 1);
+    assert(r.second);
+
+    r = db.insert_front(11, 15, 0);
+    assert(!r.second);
+
+    // This extends the segment.  At this point, 0 - 15 should have a value of 1.
+    r = db.insert_front(11, 15, 1);
+    assert(r.second);
     {
-        typedef unsigned int   key_type;
-        typedef unsigned short value_type;
-        for (value_type i = 0; i <= 100; ++i)
-            fst_test_insert_front_back<key_type, value_type>(0, 100, i);
+        db_type::const_iterator itr = r.first;
+        assert(itr->first == 0);
+        assert(itr->second == 1);
+        ++itr;
+        assert(itr->first == 15);
+    }
+
+    r = db.insert_front(2, 4, 1);
+    assert(!r.second);
+
+    // Different value segment.  This should change the state.
+    r = db.insert_front(2, 4, 0);
+    assert(r.second);
+
+    // Ditto.
+    r = db.insert_front(2, 4, 1);
+    assert(r.second);
+
+    r = db.insert_front(1, 8, 1);
+    assert(!r.second);
+
+    // Different value segment.
+    r = db.insert_front(1, 8, 2);
+    assert(r.second);
+
+    r = db.insert_front(8, 20, 2);
+    assert(r.second);
+
+    // The 0-1 segment should still have a value of 1.  So this won't change 
+    // the state.
+    r = db.insert_front(0, 1, 1);
+    assert(!r.second);
+
+    // Partially out-of-bound segment, but this should modify the value of 0-2.
+    r = db.insert_front(-50, 2, 10);
+    assert(r.second);
+    {
+        db_type::const_iterator itr = r.first;
+        assert(itr->first == 0);
+        assert(itr->second == 10);
+        ++itr;
+        assert(itr->first == 2);
+    }
+
+    // Entirely out-of-bound.
+    r = db.insert_front(-50, -2, 20);
+    assert(!r.second);
+
+    // Likewise, partially out-of-bound at the higher end.
+    r = db.insert_front(800, 1200, 20);
+    assert(r.second);
+
+    // Entirely out-of-bound.
+    r = db.insert_front(1300, 1400, 25);
+    assert(!r.second);
+}
+
+void fst_perf_test_insert_position()
+{
+    typedef flat_segment_tree<long, bool> db_type;
+    typedef pair<db_type::const_iterator, bool> ret_type;
+    long upper = 60000;
+    {
+        StackPrinter __stack_printer__("::fst_perf_test_insert_position (front)");
+        // Much smaller upper boundary because front insertion is very slow.
+        db_type db(0, upper, false);
+        bool val = false;
+        for (long i = 0; i < upper; ++i)
+        {
+            db.insert_front(i, i+1, val);
+            val = !val;
+        }
     }
 
     {
-        typedef int   key_type;
-        typedef short value_type;
-        for (value_type i = 0; i <= 100; ++i)
-            fst_test_insert_front_back<key_type, value_type>(0, 100, i);
+        StackPrinter __stack_printer__("::fst_perf_test_insert_position (back)");
+        db_type db(0, upper, false);
+        bool val = false;
+        for (long i = 0; i < upper; ++i)
+        {
+            db.insert_back(i, i+1, val);
+            val = !val;
+        }
     }
 
     {
-        typedef long         key_type;
-        typedef unsigned int value_type;
-        for (value_type i = 0; i <= 100; ++i)
-            fst_test_insert_front_back<key_type, value_type>(0, 100, i);
+        db_type db(0, upper, false);
+        {
+            StackPrinter __stack_printer__("::fst_perf_test_insert_position (position)");
+            db_type::const_iterator itr = db.begin();
+            bool val = false;
+            for (long i = 0; i < upper; ++i)
+            {
+                ret_type ret = db.insert(itr, i, i+1, val);
+                val = !val;
+                itr = ret.first;
+            }
+        }
+        {
+            StackPrinter __stack_printer__("::fst_perf_test_insert_position (position re-insert)");
+            db_type::const_iterator itr = db.begin();
+            bool val = true;
+            for (long i = 0; i < upper; ++i)
+            {
+                ret_type ret = db.insert(itr, i, i+1, val);
+                val = !val;
+                itr = ret.first;
+            }
+        }
+    }
+}
+
+void fst_perf_test_position_search()
+{
+    typedef flat_segment_tree<long, bool> db_type;
+    typedef pair<db_type::const_iterator, bool> ret_type;
+    long upper = 60000;
+    db_type db(0, upper, false);
+
+    // Fill the leaf nodes first.
+    db_type::const_iterator itr = db.begin();
+    bool val = false;
+    for (long i = 0; i < upper; ++i)
+    {
+        ret_type ret = db.insert(itr, i, i+1, val);
+        val = !val;
+        itr = ret.first;
+    }
+
+    {
+        StackPrinter __stack_printer__("::fst_perf_test_position_search (normal)");
+        for (long i = 0; i < upper; ++i)
+        {
+            bool val;
+            ret_type ret = db.search(i, val);
+            assert(ret.second);
+        }
+    }
+
+    {
+        StackPrinter __stack_printer__("::fst_perf_test_position_search (positioned)");
+        itr = db.begin();
+        for (long i = 0; i < upper; ++i)
+        {
+            bool val;
+            ret_type ret = db.search(itr, i, val);
+            assert(ret.second);
+            itr = ret.first;
+        }
+    }
+}
+
+template<typename K, typename V>
+bool check_pos_search_result(
+    const flat_segment_tree<K, V>& db, 
+    typename flat_segment_tree<K, V>::const_iterator& itr,
+    K key, K start_expected, K end_expected, V value_expected)
+{
+    typedef flat_segment_tree<K, V> db_type;
+    typedef pair<typename db_type::const_iterator, bool> ret_type;
+
+    V _val;
+    K _start = -1, _end = -1;
+
+    ret_type r = db.search(itr, key, _val, &_start, &_end);
+
+    cout << "expected: start=" << start_expected << " end=" << end_expected << " value=" << value_expected << endl;
+    cout << "observed: start=" << _start << " end=" << _end << " value=" << _val << endl;
+
+    bool result = 
+        _start == start_expected && _end == end_expected && _val == value_expected &&
+        r.first->first == start_expected && r.first->second == value_expected;
+    itr = r.first;
+    return result;
+}
+
+void fst_test_position_search()
+{
+    StackPrinter __stack_printer__("::fst_test_position_search");
+    typedef flat_segment_tree<long, short> db_type;
+    typedef pair<db_type::const_iterator, bool> ret_type;
+    
+    db_type db(0, 100, 0);
+    db.insert_front(10, 20, 1);
+    db.insert_front(30, 50, 5);
+
+    db_type db2(-10, 10, 1);
+
+    struct {
+        long start_range;
+        long end_range;
+        short value_expected;
+    } params[] = {
+        {  0,  10, 0 },
+        { 10,  20, 1 },
+        { 20,  30, 0 },
+        { 30,  50, 5 },
+        { 50, 100, 0 }
+    };
+
+    size_t n = sizeof(params) / sizeof(params[0]);
+
+    cout << "Testing for searches with various valid and invalid iterators." << endl;
+    for (size_t i = 0; i < n; ++i)
+    {
+        for (long j = params[i].start_range; j < params[i].end_range; ++j)
+        {
+            db_type::const_iterator itr;
+            bool success = false;
+
+            // empty iterator - should fall back to normal search.
+            success = check_pos_search_result(
+                db, itr, j, params[i].start_range, params[i].end_range, params[i].value_expected);
+            assert(success);
+
+            // iterator returned from the previous search.
+            success = check_pos_search_result(
+                db, itr, j, params[i].start_range, params[i].end_range, params[i].value_expected);
+            assert(success);
+
+            // begin iterator.
+            itr = db.begin();
+            success = check_pos_search_result(
+                db, itr, j, params[i].start_range, params[i].end_range, params[i].value_expected);
+            assert(success);
+
+            // end iterator.
+            itr = db.end();
+            success = check_pos_search_result(
+                db, itr, j, params[i].start_range, params[i].end_range, params[i].value_expected);
+            assert(success);
+
+            // iterator from another container - should fall back to normal search.
+            itr = db2.begin();
+            success = check_pos_search_result(
+                db, itr, j, params[i].start_range, params[i].end_range, params[i].value_expected);
+            assert(success);
+        }
+    }
+
+    cout << "Testing for continuous searching by re-using the iteraotr from the previous search." << endl;
+    db_type::const_iterator itr;
+    short val;
+    long start = 0, end = 0;
+
+    for (size_t i = 0; i < n; ++i)
+    {
+        ret_type r = db.search(itr, end, val, &start, &end);
+        assert(start == params[i].start_range);
+        assert(end == params[i].end_range);
+        assert(val == params[i].value_expected);
+        assert(r.second);
+        itr = r.first;
+    }
+}
+
+void fst_test_min_max_default()
+{
+    typedef flat_segment_tree<long, short> db_type;
+    db_type db(0, 100, 2);
+    assert(db.min_key() == 0);
+    assert(db.max_key() == 100);
+    assert(db.default_value() == 2);
+}
+
+void fst_test_swap()
+{
+    typedef flat_segment_tree<long, int> db_type;
+    db_type db1(0, 200, 20);
+    db_type db2(20, 40, 0);
+    db1.insert_back(20, 30, 1);
+    db1.insert_back(30, 40, 2);
+    db1.insert_back(40, 50, 3);
+    db1.build_tree();
+
+    // Check the content of db1.
+    {
+        db_type::key_type   k[] = {0,  20, 30, 40, 50, 200};
+        db_type::value_type v[] = {20,  1,  2,  3, 20};
+        assert(check_leaf_nodes(db1, k, v, ARRAY_SIZE(k)));
+    }
+    assert(db1.min_key() == 0);
+    assert(db1.max_key() == 200);
+    assert(db1.default_value() == 20);
+    assert(db1.is_tree_valid());
+
+    db1.swap(db2);
+
+    // Now db2 should inherit the content of db1.
+    {
+        db_type::key_type   k[] = {0,  20, 30, 40, 50, 200};
+        db_type::value_type v[] = {20,  1,  2,  3, 20};
+        assert(check_leaf_nodes(db2, k, v, ARRAY_SIZE(k)));
+    }
+    assert(db2.min_key() == 0);
+    assert(db2.max_key() == 200);
+    assert(db2.default_value() == 20);
+    assert(db2.is_tree_valid());
+
+    // Tree search should work on db2.
+    db_type::value_type val;
+    assert(db2.search_tree(35, val));
+    assert(val == 2);
+}
+
+void fst_test_clear()
+{
+    StackPrinter __stack_printer__("::fst_test_clear");
+
+    typedef flat_segment_tree<long, int> db_type;
+    db_type db(0, 100, 42);
+    db.insert_back(0, 10, 0);
+    db.insert_back(10, 20, 1);
+    db.insert_back(20, 30, 2);
+    db.build_tree();
+
+    {
+        db_type::key_type k[] = {0, 10, 20, 30, 100};
+        db_type::value_type v[] = {0, 1, 2, 42};
+        assert(check_leaf_nodes(db, k, v, ARRAY_SIZE(k)));
+    }
+    assert(db.min_key() == 0);
+    assert(db.max_key() == 100);
+    assert(db.default_value() == 42);
+    assert(db.is_tree_valid());
+
+    db.clear();
+
+    {
+        db_type::key_type k[] = {0, 100};
+        db_type::value_type v[] = {42};
+        assert(check_leaf_nodes(db, k, v, ARRAY_SIZE(k)));
+    }
+    assert(db.min_key() == 0);
+    assert(db.max_key() == 100);
+    assert(db.default_value() == 42);
+    assert(!db.is_tree_valid());
+}
+
+void fst_test_assignment()
+{
+    StackPrinter __stack_printer__("::fst_test_assignment");
+
+    typedef flat_segment_tree<long, int> db_type;
+    db_type db1(0, 100, 42);
+    db1.insert_back(0, 10, 0);
+    db1.insert_back(10, 20, 1);
+    db1.insert_back(20, 30, 2);
+    db1.build_tree();
+
+    {
+        db_type::key_type k[] = {0, 10, 20, 30, 100};
+        db_type::value_type v[] = {0, 1, 2, 42};
+        assert(check_leaf_nodes(db1, k, v, ARRAY_SIZE(k)));
+    }
+    assert(db1.min_key() == 0);
+    assert(db1.max_key() == 100);
+    assert(db1.default_value() == 42);
+    assert(db1.is_tree_valid());
+
+    db_type db2(20, 40, 0);
+    db2.insert_back(20, 30, 8);
+    db2.build_tree();
+
+    {
+        db_type::key_type k[] = {20, 30, 40};
+        db_type::value_type v[] = {8, 0};
+        assert(check_leaf_nodes(db2, k, v, ARRAY_SIZE(k)));
+    }
+    assert(db2.min_key() == 20);
+    assert(db2.max_key() == 40);
+    assert(db2.default_value() == 0);
+    assert(db2.is_tree_valid());
+
+    db_type db3(10, 80, 4);
+    db3.build_tree();
+
+    {
+        db_type::key_type k[] = {10, 80};
+        db_type::value_type v[] = {4};
+        assert(check_leaf_nodes(db3, k, v, ARRAY_SIZE(k)));
+    }
+    assert(db3.min_key() == 10);
+    assert(db3.max_key() == 80);
+    assert(db3.default_value() == 4);
+    assert(db3.is_tree_valid());
+
+    db3 = db2 = db1;
+
+    {
+        db_type::key_type k[] = {0, 10, 20, 30, 100};
+        db_type::value_type v[] = {0, 1, 2, 42};
+        assert(check_leaf_nodes(db1, k, v, ARRAY_SIZE(k)));
+    }
+    assert(db1.min_key() == 0);
+    assert(db1.max_key() == 100);
+    assert(db1.default_value() == 42);
+    assert(db1.is_tree_valid());
+
+    {
+        db_type::key_type k[] = {0, 10, 20, 30, 100};
+        db_type::value_type v[] = {0, 1, 2, 42};
+        assert(check_leaf_nodes(db2, k, v, ARRAY_SIZE(k)));
+    }
+    assert(db2.min_key() == 0);
+    assert(db2.max_key() == 100);
+    assert(db2.default_value() == 42);
+    assert(!db2.is_tree_valid());
+
+    {
+        db_type::key_type k[] = {0, 10, 20, 30, 100};
+        db_type::value_type v[] = {0, 1, 2, 42};
+        assert(check_leaf_nodes(db3, k, v, ARRAY_SIZE(k)));
+    }
+    assert(db3.min_key() == 0);
+    assert(db3.max_key() == 100);
+    assert(db3.default_value() == 42);
+    assert(!db3.is_tree_valid());
+}
+
+int main (int argc, char **argv)
+{
+    cmd_options opt;
+    if (!parse_cmd_options(argc, argv, opt))
+        return EXIT_FAILURE;
+
+    if (opt.test_func)
+    {
+        fst_test_equality();
+        fst_test_copy_ctor();
+        fst_test_back_insert();
+        {
+            typedef unsigned int   key_type;
+            typedef unsigned short value_type;
+            for (value_type i = 0; i <= 100; ++i)
+                fst_test_insert_front_back<key_type, value_type>(0, 100, i);
+        }
+    
+        {
+            typedef int   key_type;
+            typedef short value_type;
+            for (value_type i = 0; i <= 100; ++i)
+                fst_test_insert_front_back<key_type, value_type>(0, 100, i);
+        }
+    
+        {
+            typedef long         key_type;
+            typedef unsigned int value_type;
+            for (value_type i = 0; i <= 100; ++i)
+                fst_test_insert_front_back<key_type, value_type>(0, 100, i);
+        }
+    
+        fst_test_leaf_search();
+        fst_test_tree_build();
+        fst_test_tree_search();
+        fst_test_insert_search_mix();
+        fst_test_shift_left();
+        fst_test_shift_left_right_edge();
+        fst_test_shift_left_append_new_segment();
+        fst_test_shift_right_init0();
+        fst_test_shift_right_init999();
+        fst_test_shift_right_bool();
+        fst_test_shift_right_skip_start_node();
+        fst_test_shift_right_all_nodes();
+        fst_test_const_iterator();
+        fst_test_insert_iterator();
+        fst_test_insert_state_changed();
+        fst_test_position_search();
+        fst_test_min_max_default();
+        fst_test_swap();
+        fst_test_clear();
+        fst_test_assignment();
+    }
+
+    if (opt.test_perf)
+    {
+        fst_perf_test_search(true);
+        fst_perf_test_search(false);
+        fst_perf_test_insert_front_back();
+        fst_perf_test_insert_position();
+        fst_perf_test_position_search();
     }
 
-    fst_test_leaf_search();
-    fst_test_tree_build();
-    fst_test_tree_search();
-//  fst_perf_test_search(false);
-    fst_perf_test_search(true);
-    fst_test_insert_search_mix();
-    fst_test_shift_segment_left();
-    fst_test_shift_segment_left_right_edge();
-    fst_test_shift_segment_left_append_new_segment();
-    fst_test_shift_segment_right_init0();
-    fst_test_shift_segment_right_init999();
-    fst_test_shift_segment_right_bool();
-    fst_test_shift_segment_right_skip_start_node();
-    fst_test_const_iterator();
     fprintf(stdout, "Test finished successfully!\n");
     return 0;
 }
