@@ -25,8 +25,8 @@
  *
  ************************************************************************/
 
-#include "mdds/mixed_type_matrix.hpp"
 #include "test_global.hpp"
+#include "mdds/mixed_type_matrix.hpp"
 
 #include <sstream>
 #include <cassert>
@@ -263,11 +263,16 @@ void mtm_test_resize(matrix_density_t density)
     assert(mxsize.first == 6);
     assert(mxsize.second == 4);
     mx.resize(6, 6);
+    mx.set_boolean(5, 5, true);
     mx.dump();
     mxsize = mx.size();
     assert(mxsize.first == 6);
     assert(mxsize.second == 6);
     mx.resize(3, 6);
+    mx.set_numeric(2, 2, 4.2);
+    mx.set_numeric(2, 3, 3.2);
+    mx.set_numeric(2, 4, 2.2);
+    mx.set_numeric(2, 5, 1.2);
     mx.dump();
     mxsize = mx.size();
     assert(mxsize.first == 3);
@@ -690,15 +695,16 @@ void traverse_itr_access(typename _StoreType::const_itr_access& itr_access)
     assert(i == 0);
 }
 
+template<typename _FilledStoreType>
 void mtm_test_iterator_access_filled(size_t rows, size_t cols)
 {
     StackPrinter __stack_printer__("::mtm_test_iterator_access_filled");
-    typedef storage_filled<mx_type> store_type;
+    typedef _FilledStoreType store_type;
 
     store_type store(rows, cols, matrix_init_element_zero);
     {
         cout << "rows: " << rows << "  cols: " << cols << endl;
-        store_type::const_itr_access* itr_access = store.get_const_itr_access();
+        typename store_type::const_itr_access* itr_access = store.get_const_itr_access();
         traverse_itr_access<store_type>(*itr_access);
         delete itr_access;
     }
@@ -839,6 +845,48 @@ void mtm_test_const_iterator()
     assert(itr == itr_end); // not found.
 }
 
+/**
+ * Measure the performance of object instantiation for filled storage.
+ */
+void mtm_perf_test_filled_storage_creation()
+{
+    StackPrinter __stack_printer__("::mtm_perf_test_filled_storage_creation");
+    cout << "measuring performance on matrix object creation." << endl;
+    size_t rowsize = 1000;
+    size_t obj_count = 30000;
+    cout << "row size: " << rowsize << "  object count: " << obj_count << endl;
+    for (size_t colsize = 1; colsize <= 3; ++colsize)
+    {
+        StackPrinter __stack_printer2__("::mtm_perf_test_filled_storage_creation::group");
+        cout << "column size: " << colsize << endl;
+        for (size_t i = 0; i < obj_count; ++i)
+            mx_type mx(rowsize, colsize, matrix_density_filled_zero);
+    }
+}
+
+void mtm_perf_test_filled_storage_set_numeric()
+{
+    StackPrinter __stack_printer__("::mtm_perf_test_filled_storage_set_numeric");
+    cout << "measuring performance on matrix object creation and populating it with numeric data." << endl;
+    size_t rowsize = 1000;
+    size_t obj_count = 30000;
+    cout << "row size: " << rowsize << "  object count: " << obj_count << endl;
+    for (size_t colsize = 1; colsize <= 3; ++colsize)
+    {
+        StackPrinter __stack_printer2__("::mtm_perf_test_filled_storage_set_numeric::group");
+        cout << "column size: " << colsize << endl;
+        for (size_t i = 0; i < obj_count; ++i)
+        {
+            mx_type mx(rowsize, colsize, matrix_density_filled_zero);
+            for (size_t row = 0; row < rowsize; ++row)
+            {
+                for (size_t col = 0; col < colsize; ++col)
+                    mx.set_numeric(row, col, 1.0);
+            }
+        }
+    }
+}
+
 int main(int argc, char** argv)
 {
     cmd_options opt;
@@ -861,16 +909,35 @@ int main(int argc, char** argv)
     
         run_tests_on_all_density_types(mtm_test_flag_storage);
     
-        mtm_test_iterator_access_filled(1, 1);
-        mtm_test_iterator_access_filled(3, 1);
-        mtm_test_iterator_access_filled(1, 3);
-        mtm_test_iterator_access_filled(3, 3);
-        mtm_test_iterator_access_filled(0, 0);
+        mtm_test_iterator_access_filled<storage_filled_nested_array<mx_type> >(1, 1);
+        mtm_test_iterator_access_filled<storage_filled_nested_array<mx_type> >(3, 1);
+        mtm_test_iterator_access_filled<storage_filled_nested_array<mx_type> >(1, 3);
+        mtm_test_iterator_access_filled<storage_filled_nested_array<mx_type> >(3, 3);
+        mtm_test_iterator_access_filled<storage_filled_nested_array<mx_type> >(0, 0);
+
+        mtm_test_iterator_access_filled<storage_filled_linear<mx_type> >(1, 1);
+        mtm_test_iterator_access_filled<storage_filled_linear<mx_type> >(3, 1);
+        mtm_test_iterator_access_filled<storage_filled_linear<mx_type> >(1, 3);
+        mtm_test_iterator_access_filled<storage_filled_linear<mx_type> >(3, 3);
+        mtm_test_iterator_access_filled<storage_filled_linear<mx_type> >(0, 0);
+
+        mtm_test_iterator_access_filled<storage_filled_linear_zero<mx_type> >(1, 1);
+        mtm_test_iterator_access_filled<storage_filled_linear_zero<mx_type> >(3, 1);
+        mtm_test_iterator_access_filled<storage_filled_linear_zero<mx_type> >(1, 3);
+        mtm_test_iterator_access_filled<storage_filled_linear_zero<mx_type> >(3, 3);
+        mtm_test_iterator_access_filled<storage_filled_linear_zero<mx_type> >(0, 0);
     
         mtm_test_iterator_access_sparse();
     
         mtm_test_const_iterator();
     }
+
+    if (opt.test_perf)
+    {
+        mtm_perf_test_filled_storage_creation();
+        mtm_perf_test_filled_storage_set_numeric();
+    }
+
     cout << "Test finished successfully!" << endl;
     return EXIT_SUCCESS;
 }
